@@ -6,6 +6,7 @@ import subprocess
 from io import BytesIO
 from itertools import product as cartesian_product
 from pathlib import Path
+from typing import cast
 from unittest import mock
 
 import pytest
@@ -30,6 +31,7 @@ from pypdf.generic import (
     NameObject,
     NullObject,
     NumberObject,
+    StreamObject,
 )
 
 from . import PILContext, get_data_from_url
@@ -790,3 +792,27 @@ def test_jbig2decode__edge_cases(caplog):
         "jbig2dec FATAL ERROR page has no image, cannot be completed",
         "jbig2dec WARNING unable to complete page"
     ]
+
+
+@pytest.mark.timeout(timeout=30, method="thread")
+@pytest.mark.enable_socket
+def test_flate_decode_stream_with_faulty_tail_bytes():
+    """
+    Test for #3332
+
+    The test ensures two things:
+        1. stream can be decoded at all
+        2. decoding doesn't falls through to last fallback in try-except blocks
+           that is too slow and takes ages for this stream
+    """
+    data = get_data_from_url(
+        url="https://github.com/user-attachments/files/20901522/faulty_stream_tail_example.1.pdf",
+        name="faulty_stream_tail_example.1.pdf"
+    )
+    expected = get_data_from_url(
+        url="https://github.com/user-attachments/files/20941717/decoded.dat.txt",
+        name="faulty_stream_tail_example.1.decoded.dat"
+    )
+    reader = PdfReader(BytesIO(data))
+    obj = reader.get_object(IndirectObject(182, 0, reader))
+    assert cast(StreamObject, obj).get_data() == expected
